@@ -780,13 +780,8 @@ roccat_probe(struct ratbag_device *device)
 	int active_idx;
 
 	rc = ratbag_open_hidraw(device);
-	if (rc) {
-		log_error(device->ratbag,
-			  "Can't open corresponding hidraw node: '%s' (%d)\n",
-			  strerror(-rc),
-			  rc);
-		return -ENODEV;
-	}
+	if (rc)
+		return rc;
 
 	if (!ratbag_hidraw_has_report(device, ROCCAT_REPORT_ID_KEY_MAPPING)) {
 		ratbag_close_hidraw(device);
@@ -808,8 +803,8 @@ roccat_probe(struct ratbag_device *device)
 	if (active_idx < 0) {
 		log_error(device->ratbag,
 			  "Can't talk to the mouse: '%s' (%d)\n",
-			  strerror(-rc),
-			  rc);
+			  strerror(-active_idx),
+			  active_idx);
 		rc = -ENODEV;
 		goto err;
 	}
