@@ -34,6 +34,7 @@
 #include <unistd.h>
 #include <limits.h>
 
+#include "usb-ids.h"
 #include "libratbag-private.h"
 #include "libratbag-util.h"
 
@@ -283,23 +284,15 @@ out:
 	return rc;
 }
 
-bool
-ratbag_assign_driver(struct ratbag_device *device,
-		     const struct input_id *dev_id,
-		     struct ratbag_test_device *test_device)
+static inline bool
+ratbag_test_driver(struct ratbag_device *device,
+		   const struct input_id *dev_id,
+		   const char *driver_name,
+		   struct ratbag_test_device *test_device)
 {
 	struct ratbag *ratbag = device->ratbag;
 	struct ratbag_driver *driver;
 	int rc;
-	const char *driver_name;
-
-	if (!test_device) {
-		driver_name = udev_prop_value(device->udev_device, "RATBAG_DRIVER");
-		if (!driver_name)
-			return false;
-	} else {
-		driver_name = "test_driver";
-	}
 
 	list_for_each(driver, &ratbag->drivers, link) {
 		if (streq(driver->id, driver_name)) {
@@ -311,7 +304,7 @@ ratbag_assign_driver(struct ratbag_device *device,
 	if (!device->driver) {
 		log_error(ratbag, "%s: driver '%s' does not exist\n",
 			  device->name, driver_name);
-		return false;
+		goto error;
 	}
 
 	if (test_device)
@@ -320,21 +313,64 @@ ratbag_assign_driver(struct ratbag_device *device,
 		rc = device->driver->probe(device);
 	if (rc == 0) {
 		if (!ratbag_sanity_check_device(device)) {
-			return false;
+			goto error;
 		} else {
-			log_debug(ratbag, "driver match found: %s\n", driver->name);
+			log_debug(ratbag,
+				  "driver match found: %s\n",
+				  device->driver->name);
 			return true;
 		}
 	}
 
+	if (rc != -ENODEV)
+		log_error(ratbag, "%s: error opening hidraw node (%s)\n",
+			  device->name, strerror(-rc));
+
+error:
 	device->driver = NULL;
 
-	if (rc == -ENODEV)
-		log_error(ratbag, "%s: no hidraw device found\n", device->name);
-
 	return false;
 }
 
+static inline bool
+ratbag_driver_fallback_logitech(struct ratbag_device *device,
+				const struct input_id *dev_id)
+{
+	int rc;
+
+	if (dev_id->vendor != USB_VENDOR_ID_LOGITECH)
+		return false;
+
+	rc = ratbag_test_driver(device, dev_id, "hidpp20", NULL);
+	if (!rc)
+		rc = ratbag_test_driver(device, dev_id, "hidpp10", NULL);
+
+	return rc;
+}
+
+bool
+ratbag_assign_driver(struct ratbag_device *device,
+		     const struct input_id *dev_id,
+		     struct ratbag_test_device *test_device)
+{
+	bool rc;
+	const char *driver_name;
+
+	if (!test_device) {
+		driver_name = udev_prop_value(device->udev_device, "RATBAG_DRIVER");
+	} else {
+		driver_name = "test_driver";
+	}
+
+	if (driver_name) {
+		rc = ratbag_test_driver(device, dev_id, driver_name, test_device);
+	} else {
+		rc = ratbag_driver_fallback_logitech(device, dev_id);
+	}
+
+	return rc;
+}
+
 static inline char*
 get_device_name(struct udev_device *device)
 {
@@ -382,6 +418,9 @@ ratbag_device_new_from_udev_device(struct ratbag *ratbag,
 	assert(udev_device != NULL);
 	assert(device_out != NULL);
 
+	if (udev_prop_value(udev_device, "ID_INPUT_MOUSE") == NULL)
+		goto out_err;
+
 	if (get_product_id(udev_device, &id) != 0)
 		goto out_err;
 
